@@ -29,10 +29,15 @@
 
 #include <gio/gappinfo.h>
 #include <gio/gasyncresult.h>
+#include <gio/gasyncinitable.h>
 #include <gio/gbufferedinputstream.h>
 #include <gio/gbufferedoutputstream.h>
 #include <gio/gcancellable.h>
 #include <gio/gcontenttype.h>
+#include <gio/gconverter.h>
+#include <gio/gconverterinputstream.h>
+#include <gio/gconverteroutputstream.h>
+#include <gio/gcharsetconverter.h>
 #include <gio/gdatainputstream.h>
 #include <gio/gdataoutputstream.h>
 #include <gio/gdrive.h>
@@ -56,7 +61,10 @@
 #include <gio/gioenumtypes.h>
 #include <gio/gioerror.h>
 #include <gio/giomodule.h>
+#include <gio/ginitable.h>
 #include <gio/gioscheduler.h>
+#include <gio/giostream.h>
+#include <gio/gfileiostream.h>
 #include <gio/gloadableicon.h>
 #include <gio/gmemoryinputstream.h>
 #include <gio/gmemoryoutputstream.h>
@@ -68,15 +76,41 @@
 #include <gio/goutputstream.h>
 #include <gio/gresolver.h>
 #include <gio/gseekable.h>
+#include <gio/gsettings.h>
 #include <gio/gsimpleasyncresult.h>
+#include <gio/gsocket.h>
 #include <gio/gsocketaddress.h>
 #include <gio/gsocketaddressenumerator.h>
+#include <gio/gsocketclient.h>
 #include <gio/gsocketconnectable.h>
+#include <gio/gsocketconnection.h>
+#include <gio/gsocketcontrolmessage.h>
+#include <gio/gsocketlistener.h>
+#include <gio/gsocketservice.h>
+#include <gio/gtcpconnection.h>
+#include <gio/gthreadedsocketservice.h>
 #include <gio/gsrvtarget.h>
 #include <gio/gthemedicon.h>
 #include <gio/gvfs.h>
 #include <gio/gvolume.h>
 #include <gio/gvolumemonitor.h>
+#include <gio/gzlibcompressor.h>
+#include <gio/gzlibdecompressor.h>
+
+#include <gio/gdbusutils.h>
+#include <gio/gdbusaddress.h>
+#include <gio/gdbusmessage.h>
+#include <gio/gdbusconnection.h>
+#include <gio/gdbuserror.h>
+#include <gio/gdbusnameowning.h>
+#include <gio/gdbusnamewatching.h>
+#include <gio/gdbusproxywatching.h>
+#include <gio/gdbusproxy.h>
+#include <gio/gdbusintrospection.h>
+#include <gio/gdbusmethodinvocation.h>
+#include <gio/gdbusserver.h>
+#include <gio/gcredentials.h>
+#include <gio/gdbusauthobserver.h>
 
 #undef __GIO_GIO_H_INSIDE__
 
