@@ -38,14 +38,28 @@
 #include "glocalfileinfo.h"
 #include "glibintl.h"
 
+#ifdef G_OS_UNIX
+#include "gfiledescriptorbased.h"
+#endif
+
 #ifdef G_OS_WIN32
 #include <io.h>
 #endif
 
 #include "gioalias.h"
 
+
+#ifdef G_OS_UNIX
+static void       g_file_descriptor_based_iface_init   (GFileDescriptorBasedIface *iface);
+#endif
+
 #define g_local_file_input_stream_get_type _g_local_file_input_stream_get_type
-G_DEFINE_TYPE (GLocalFileInputStream, g_local_file_input_stream, G_TYPE_FILE_INPUT_STREAM);
+G_DEFINE_TYPE_WITH_CODE (GLocalFileInputStream, g_local_file_input_stream, G_TYPE_FILE_INPUT_STREAM,
+#ifdef G_OS_UNIX
+			 G_IMPLEMENT_INTERFACE (G_TYPE_FILE_DESCRIPTOR_BASED,
+						g_file_descriptor_based_iface_init)
+#endif
+);
 
 struct _GLocalFileInputStreamPrivate {
   int fd;
@@ -75,6 +89,9 @@ static GFileInfo *g_local_file_input_stream_query_info (GFileInputStream  *strea
 							const char        *attributes,
 							GCancellable      *cancellable,
 							GError           **error);
+#ifdef G_OS_UNIX
+static int        g_local_file_input_stream_get_fd     (GFileDescriptorBased *stream);
+#endif
 
 static void
 g_local_file_input_stream_finalize (GObject *object)
@@ -109,6 +126,14 @@ g_local_file_input_stream_class_init (GLocalFileInputStreamClass *klass)
   file_stream_class->query_info = g_local_file_input_stream_query_info;
 }
 
+#ifdef G_OS_UNIX
+static void
+g_file_descriptor_based_iface_init (GFileDescriptorBasedIface *iface)
+{
+  iface->get_fd = g_local_file_input_stream_get_fd;
+}
+#endif
+
 static void
 g_local_file_input_stream_init (GLocalFileInputStream *info)
 {
@@ -118,12 +143,6 @@ g_local_file_input_stream_init (GLocalFileInputStream *info)
   info->priv->do_close = TRUE;
 }
 
-/**
- * g_local_file_input_stream_new:
- * @fd: File Descriptor.
- * 
- * Returns: #GFileInputStream for the given file descriptor.
- **/
 GFileInputStream *
 _g_local_file_input_stream_new (int fd)
 {
@@ -342,3 +361,12 @@ g_local_file_input_stream_query_info (GFileInputStream  *stream,
 					 attributes,
 					 error);
 }
+
+#ifdef G_OS_UNIX
+static int
+g_local_file_input_stream_get_fd (GFileDescriptorBased *fd_based)
+{
+  GLocalFileInputStream *stream = G_LOCAL_FILE_INPUT_STREAM (fd_based);
+  return stream->priv->fd;
+}
+#endif
